@@ -4,7 +4,9 @@
 # unittest2 is Copyright (c) 2001-2010 Python Software Foundation; All
 # Rights Reserved. See: http://docs.python.org/license.html
 
+import sys
 import logging
+import unittest
 
 import argparse
 import six
@@ -40,13 +42,17 @@ class PluginMeta(type):
             'always-on', default=instance.alwaysOn)
 
         instance.__init__(*args, **kwargs)
+
         if always_on:
             instance.register()
-        else:
-            if switch is not None:
-                short_opt, long_opt, help = switch
-                instance.addOption(
-                    instance._register_cb, short_opt, long_opt, help)
+
+        if switch is not None:
+            short_opt, long_opt, help = switch
+            if always_on:  # always-on plugins should hide their options
+                help = argparse.SUPPRESS
+            instance.addOption(
+                instance._register_cb, short_opt, long_opt, help)
+
         return instance
 
 
@@ -127,7 +133,8 @@ class Plugin(six.with_metaclass(PluginMeta)):
                     self.session.hooks.register(method, plugin)
 
     def _register_cb(self, *_):
-        self.register()
+        if not self.registered:
+            self.register()
 
     def addFlag(self, callback, short_opt, long_opt, help_text=None):
         """Add command-line flag that takes no arguments
@@ -343,10 +350,15 @@ class Event(object):
                           for k in self._attrs])
 
     def __getstate__(self):
-        state = self.__dict__
+        state = self.__dict__.copy()
         # FIXME fails for loadTestsFailure
         if 'test' in state:
-            state['test'] = util.test_name(state['test'])
+            test = state['test']
+            state['test'] = util.test_name(test)
+            # subtest support
+            if sys.version_info >= (3, 4):
+                if isinstance(test, unittest.case._SubTest):
+                    state['metadata']['subtest'] = (test._message, test.params)
         if 'executeTests' in state:
             state['executeTests'] = None
         if 'exc_info' in state and state['exc_info'] is not None:
@@ -699,7 +711,7 @@ class TestOutcomeEvent(Event):
     .. attribute :: outcome
 
        Description of test outcome. Typically will be one of 'error',
-       'failed', 'skipped', or 'passed'.
+       'failed', 'skipped', 'passed', or 'subtest'.
 
     .. attribute :: exc_info
 
